// knk_nextpeak.cpp

#include <knk_nextpeak.h>
#include <knk_normmix.h> 
#include <knk_args.h>
#undef NJS_DEBUG
#include <njs_debug.h>
#include <njs_chisquare.h>
#include <njs_function.h>
#include <math.h>

#define ITMAX 100
#define CGOLD 0.3819660
#define PI 3.1415927
#define ZEPS 1.0e-8
#define SHFT(a,b,c,d) (a)=(b);(b)=(c);(c)=(d);
#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))

using namespace Njs;

bool Knk::NextPeak::reportRegions() 
{
  assert(d_regionChrom.size() == d_regionLeftEnds.size() && d_regionLeftEnds.size() == d_regionRightEnds.size());
  if(d_regionChrom.size() == 0 ) return false;

  for(size_t i = 0; i < d_regionChrom.size(); i++) {
    std::cout<<"chromosome\t"<<d_regionChrom[i]+1
	     <<"\tcount\t"<<d_regionCnt[i]
	     <<"\tstart\t"<<d_regionLeftEnds[i]+1
	     <<"\tend\t"<<d_regionRightEnds[i]
	     <<"\tlength\t"<<d_regionRightEnds[i]-d_regionLeftEnds[i]<<std::endl;
    for(size_t j = d_regionLeftEnds[i]; j < d_regionRightEnds[i]; j++) {
      if(d_mask_p[d_regionChrom[i]][j]) {
	std::cout<<d_cntMappedTags_p[0][d_regionChrom[i]][j]<<"-"<<d_cntMappedTags_p[1][d_regionChrom[i]][j]<<" ";
      } else {
	std::cout<<"Non ";
      }
    }
    std::cout<<std::endl;
  }
  return true;
} 

Njs::SeqReadProto *Knk::NextPeak::writeRegions(const char *outFileName_)
{
  std::ofstream out (outFileName_,std::ios_base::out);
  return writeRegions(out);
}

Njs::SeqReadProto * Knk::NextPeak::writeRegions(std::ofstream &out)
{
  assert(out);
  assert(d_regionChrom.size() == d_regionLeftEnds.size() && d_regionLeftEnds.size() == d_regionRightEnds.size());

  for(size_t i = 0; i < d_regionChrom.size(); i++) {
    out<<"chromosome\t"<<d_regionChrom[i]+1
       <<"\tcount\t"<<d_regionCnt[i]
       <<"\tstart\t"<<d_regionLeftEnds[i]+1
       <<"\tend\t"<<d_regionRightEnds[i]
       <<"\tlength\t"<<d_regionRightEnds[i]-d_regionLeftEnds[i]<<"\n";
    for(size_t j = d_regionLeftEnds[i]; j < d_regionRightEnds[i]; j++) {
      if(d_mask_p[d_regionChrom[i]][j]) {
	out<<d_cntMappedTags_p[0][d_regionChrom[i]][j]<<"-"<<d_cntMappedTags_p[1][d_regionChrom[i]][j]<<" ";
      } else {
	out<<"Non ";
      }
    }
    out<<"\n";
  }
  out.close();
  return this;
} 

Njs::SeqReadProto * Knk::NextPeak::readSimRegions(const char *inFileName_)
{
  ifstream in (inFileName_, ios_base::in);
  if(! in) Njs::Io::abortFormat("'%s' not found", inFileName_);

  char *headerLineStr, *dataLineStr, *tmp;
  string line;
  long regionLen = 1000, regionCnt = 0, tagCnt = 0, j = 0;
  while(getline(in, line)) {
    char headerLineStr[line.size()];
    strcpy(headerLineStr, line.c_str());
    tmp = strtok(headerLineStr, " -\t");
    if(!strcmp(tmp, "Simulation:")) {
      d_regionChrom.push_back(0);
      d_regionLeftEnds.push_back(regionCnt * regionLen);
      d_regionRightEnds.push_back((regionCnt + 1) * regionLen);
      tagCnt = 0;
      j = 0;
    } else {
      do {
	if(!strcmp(tmp, "Non")) {
	  d_mask_p[0][regionCnt * regionLen + j] = false; 
	} else {
	  d_cntMappedTags_p[0][0][regionCnt * regionLen + j] = atoi(tmp);
	  d_cntMappedTags_p[1][0][regionCnt * regionLen + j] = atoi(strtok(NULL, " -\t"));
	  tagCnt += (d_cntMappedTags_p[0][0][regionCnt * regionLen + j] + d_cntMappedTags_p[1][0][regionCnt * regionLen + j]);
	}
	tmp = strtok(NULL, " -\t"); 
	j++;
      } while(tmp);
      regionCnt++;
      d_regionCnt.push_back(tagCnt);
    }
  }
  return this;
} 

void Knk::NextPeak::findRegions()
{
  long regionLeft, regionRight, intrinsicLeft, intrinsicRight, intrinsicRange, mean, sum, n, seqLim, loc;
  bool prevCntResult;
  for(size_t chm = 0; chm < d_seqNum; chm++) { //chm starts from 0
    prevCntResult = true;
    seqLim = d_seqLen_p[chm] - d_winSize;
    for(size_t loc = 0; loc < seqLim; loc++) { //loc starts from 0 
      if(!prevCntResult) { 
	cntNext();
      } else {
	cnt(chm, loc); 
      }
      if(d_curCnt >= d_minReadsPerRegion) {
	prevCntResult = true;
	regionLeft = d_curLeft;
	regionRight = d_curRight;
	while(loc < seqLim) {
	  loc++;
	  if( cntNext() < d_minReadsPerRegion) {
	    regionRight = d_curRight - 1;
	    break;
	  }
	}
	intrinsicLeft = regionLeft;
	intrinsicRight = regionRight;
	while(d_cntMappedTags_p[0][d_curChrom][intrinsicLeft] == 0 && d_cntMappedTags_p[1][d_curChrom][intrinsicLeft] == 0) {
	  intrinsicLeft++;
	}	
	while(d_cntMappedTags_p[0][d_curChrom][intrinsicRight-1] == 0 && d_cntMappedTags_p[1][d_curChrom][intrinsicRight-1] == 0) {
	  intrinsicRight--;
	}
	intrinsicRange = intrinsicRight - intrinsicLeft;
	if(intrinsicRange >= d_winSize) {
	  regionLeft = intrinsicLeft;
	  regionRight = intrinsicRight;
	} else {
	  sum = 0; n = 0;
	  for(size_t i = intrinsicLeft; i < intrinsicRight; i++) {
	    n += d_cntMappedTags_p[0][d_curChrom][i] + d_cntMappedTags_p[1][d_curChrom][i];
	    sum += (d_cntMappedTags_p[0][d_curChrom][i] + d_cntMappedTags_p[1][d_curChrom][i])*(i-intrinsicLeft);
	  }
	  mean = sum/n + intrinsicLeft;
	  if(mean - intrinsicLeft - (intrinsicRight - mean) > d_winSize - intrinsicRange
	     || intrinsicRight - mean - (mean - intrinsicLeft) > d_winSize - intrinsicRange) {
	    if(mean - intrinsicLeft > intrinsicRight - mean) {
	      regionLeft = intrinsicLeft;
	      regionRight = regionLeft + d_winSize;
	    } else {
	      regionRight = intrinsicRight;
	      regionLeft = regionRight - d_winSize;
	    }
	  } else {
	    if(regionLeft < mean - d_winSize* 0.5) {
	       regionLeft = mean - d_winSize* 0.5;
	       regionRight = regionLeft + d_winSize;
	    } else {
	      regionRight = mean + d_winSize* 0.5;
	    }
	  }
	}
	
	d_regionChrom.push_back(chm);
        d_regionLeftEnds.push_back(regionLeft);
	d_regionRightEnds.push_back(regionRight);
	d_regionCnt.push_back(cnt(chm, regionLeft, regionRight));
      	loc = regionRight -1 ;
      } else { prevCntResult = false; }	
    } // for loop with loc
  }
}

long Knk::NextPeak::cnt(
   const long chrom_,
   const long left_)
{
   d_curChrom = chrom_;
   d_curLeft = left_;
   d_curRight = left_ + d_winSize;

   d_curCnt = 0;
   for(size_t i = 0; i < 2; i++)
      for(size_t loc = left_; loc < d_curRight; loc++)
         d_curCnt += d_cntMappedTags_p[i][chrom_][loc];
   return d_curCnt;
}

long Knk::NextPeak::cnt(
   const long chrom_,
   const long left_,
   const long right_)
{
   d_curChrom = chrom_;
   d_curLeft = left_;
   d_curRight = right_;

   d_curCnt = 0;
   for(size_t i = 0; i < 2; i++)
      for(size_t loc = left_; loc < right_; loc++)
         d_curCnt += d_cntMappedTags_p[i][chrom_][loc];
   return d_curCnt;
}

long Knk::NextPeak::cntNext()
{
   d_curCnt += (d_cntMappedTags_p[0][d_curChrom][d_curRight] + d_cntMappedTags_p[1][d_curChrom][d_curRight] 
      - d_cntMappedTags_p[0][d_curChrom][d_curLeft] - d_cntMappedTags_p[1][d_curChrom][d_curLeft]);
   d_curLeft++;
   d_curRight++;
   return d_curCnt;
}

long Knk::NextPeak::cntPrev()
{
   d_curCnt += (d_cntMappedTags_p[0][d_curChrom][d_curLeft-1] + d_cntMappedTags_p[1][d_curChrom][d_curLeft-1]
      - d_cntMappedTags_p[0][d_curChrom][d_curRight-1] - d_cntMappedTags_p[1][d_curChrom][d_curRight-1]);
   d_curLeft--;
   d_curRight--;
   return d_curCnt;
}

long Knk::NextPeak::cntIncreasingWin()
{
   d_curCnt += (d_cntMappedTags_p[0][d_curChrom][d_curRight] + d_cntMappedTags_p[1][d_curChrom][d_curRight]);
   d_curRight++;
   return d_curCnt;

}

SeqReadProto *Knk::NextPeak::setWinSizeMinTagNum(
   long winSize_, 	
   long minReadsPerRegion_)
{
   d_winSize = winSize_;
   d_minReadsPerRegion = minReadsPerRegion_;
   return this;
}

SeqReadProto *Knk::NextPeak::setChromNum() 
{
  if(d_chromNum_p == 0 && getStoreDefLine() ) {
    d_seqNum = getCountSeqs();
    d_chromNum_p = new long [d_seqNum];
    char tmp[2], b[3];
    char strch[] = "ch",strgi[] = "gi", strX[] = "X", strY[] = "Y";
    tmp[0] = getDefLine(0)[1];
    tmp[1] = getDefLine(0)[2]; 
    b[0] = tmp[0]; b[1] = tmp[1]; b[2] = '\0';

     if(strcmp(b,strgi) == 0) { // form of ">gi|89161185|ref|NC_000001.9|NC_000001 Homo sapiens chromosome 1, reference assembly, complete sequence"
       char a[256];
       for(size_t i = 0; i < d_seqNum ; i++) {
	 strcpy(a, getDefLine(i));
	 strtok(a, " _.\t");
	 d_chromNum_p[i] = atoi(strtok(NULL, " _.\t")) - 1;
       }
    } else if(strcmp(b,strch) == 0) { // form of ">chr1"
      for(size_t i = 0; i < d_seqNum ; i++) {
	tmp[0] = getDefLine (i) [1+3];
	tmp[1] = getDefLine (i) [2+3];
	b[0] = tmp[0]; b[1] = tmp[1]; b[2] = '\0';
	if(strcmp(b, strX) == 0 ) {  // works only for human genome
	  d_chromNum_p[i] = 22;
	} else if(strcmp(b, strY) == 0) {
	  d_chromNum_p[i] = 23;
	} else {
	  d_chromNum_p[i] = atoi(b) - 1;
	}
      }      
    } else { // form of ">1"
      for(size_t i = 0; i < d_seqNum ; i++) {
	b[0] = getDefLine (i) [1];
	b[1] = getDefLine (i) [2];
	d_chromNum_p[i] = atoi(b) - 1;
      }
    }
  }
   return this;
}

SeqReadProto *Knk::NextPeak::setStrIndexFromChrom() 
{
  if(d_strIndexFromChrom_p == 0) {
    d_strIndexFromChrom_p = new long [d_seqNum];
    for(size_t chm = 0; chm < d_seqNum; chm++) {
      for(size_t j = 0; j < d_seqNum; j++) {
	if(d_chromNum_p[j] == chm) {
	  d_strIndexFromChrom_p[chm] = j;
	  continue;
	}
      }
    }
  }
  return this;
}

SeqReadProto *Knk::NextPeak::readSeqLenFile(const char* inFileName_) 
{
  std::ifstream in (inFileName_, std::ios_base::in);
  if(! in) Io::abortFormat("'%s' not found", inFileName_);  

  std::vector <long> seqLen;
  long tmp;
  while(in >>tmp) {
    seqLen.push_back(tmp);
  }
  d_seqNum = seqLen.size();
  d_seqLen_p = new long [d_seqNum];
  for(size_t i = 0; i < d_seqNum; i++) {
    d_seqLen_p[i] = seqLen[i];
  }

  return this;
}

void Knk::NextPeak::useHg18SeqLen()
{
  d_seqNum = 24;
  long seqLen[] = {247249719,242951149,199501827,191273063,180857866,170899992,158821424,146274826,140273252,135374737,134452384,132349534,114142980,106368585,100338915,88827254,78774742,76117153,63811651,62435964,46944323,49691432,154913754,57772954};
  d_seqLen_p = new long [d_seqNum];
  for(size_t i = 0 ; i < d_seqNum; i++ )
    d_seqLen_p[i] = seqLen[i];
}

SeqReadProto *Knk::NextPeak::clearChipSeqMappedTags ()
{
  if(d_cntMappedTags_p != 0) {
   for(size_t i = 0; i < 2; i++) {
     for(size_t j = 0; j < d_seqNum; j++) 
       delete [] d_cntMappedTags_p[i][j];
     delete [] d_cntMappedTags_p[i];
   }
   delete [] d_cntMappedTags_p; d_cntMappedTags_p = 0;
  }
  return this;
}

SeqReadProto *Knk::NextPeak::free ()
{
  Knk::NextPeak::clearChipSeqMappedTags();
  if(d_seqLen_p != 0) {
    delete [] d_seqLen_p; d_seqLen_p=0;
  }
  if(d_chromNum_p != 0) {
    delete [] d_chromNum_p; d_chromNum_p=0;
  }
  if(d_strIndexFromChrom_p != 0) {
    delete [] d_strIndexFromChrom_p; d_strIndexFromChrom_p=0;
  }

  if(d_mask_p != 0 && d_seqNum != 0) {
    for(size_t i = 0; i < d_seqNum; i++)
      delete [] d_mask_p[i];
    delete [] d_mask_p; d_mask_p = 0;
  }

  d_tooManyMatchRead.clear();    
  d_regionChrom.clear();
  d_regionLeftEnds.clear();
  d_regionRightEnds.clear();
  d_regionCnt.clear();
  for(size_t i=0; i < d_cntAnchored.size(); i++)
    d_cntAnchored[i].clear();
  d_cntAnchored.clear();
  d_motifChm.clear();
  d_motifLoc.clear();
  return this;
}


void Knk::NextPeak::readMotifLocReportAnchoredLoc(
					      const char *inFileName_, 
					      const long width_,
					      const long shift_,
					      const char *outFileName_)
{
  std::ifstream in (inFileName_, std::ios_base::in);
  if(! in) Io::abortFormat("'%s' not found", inFileName_);  

  long tmp;
  while(in >>tmp) {
    d_motifChm.push_back(tmp-1);
    in >>tmp;
    d_motifLoc.push_back(tmp-1-shift_);
  }

  const long halfwid = width_ * 0.5;
  d_cntAnchored.resize(2);
  d_cntAnchored[0].resize(width_); 
  d_cntAnchored[1].resize(width_);
  if(d_motifLoc[1] - d_motifLoc[0] > 2 * width_) {
    for(size_t i = 0; i < 2; i++) 
      for(size_t j = 0; j < width_ && d_motifLoc[0]-halfwid+j >= 0 ; j++)
	d_cntAnchored[i][j] += d_cntMappedTags_p[i][d_motifChm[0]][d_motifLoc[0]-halfwid+j];
     
  }

  for(size_t k = 1; k < d_motifChm.size()- 1; k++) {
    if(d_motifLoc[k] - d_motifLoc[k-1] > 2 * width_ && d_motifLoc[k+1] - d_motifLoc[k] > 2 * width_) {
      for(size_t i = 0; i < 2; i++) 
	for(size_t j = 0; j < width_ && d_motifLoc[k]-halfwid+j >= 0; j++)
	  d_cntAnchored[i][j] += d_cntMappedTags_p[i][d_motifChm[k]][d_motifLoc[k]-halfwid+j];
    }    
  }
  if(d_motifLoc[d_motifChm.size()-1] - d_motifLoc[d_motifChm.size()-2] > 2 * width_) {
    for(size_t i = 0; i < 2; i++) 
      for(size_t j = 0; j < width_; j++)
	d_cntAnchored[i][j] += d_cntMappedTags_p[i][d_motifChm[d_motifChm.size()-1]][d_motifLoc[d_motifChm.size()]-halfwid+j];
  }    

  if(outFileName_) {
    std::ofstream out (outFileName_,std::ios_base::out);
    for(long j = 0; j < width_; j++)
      out<<j-halfwid<<" "<<d_cntAnchored[0][j]<<" "<<d_cntAnchored[1][j]<<"\n";
    out.close();
  }
}

SeqReadProto * Knk::NextPeak::readMappability(const char *inDirName_)
{
  std::string dirname = inDirName_;
  return readMappability(dirname);
}

SeqReadProto * Knk::NextPeak::readMappability(std::string inDirName_)
{
  std::string filename;
  char letter;
  long incompCnt = 0;
  for(size_t chm = 0; chm < d_seqNum; chm++) {
    std::stringstream ss;
    ss << chm+1;
    filename = inDirName_ + "chr" + ss.str() + ".map";
    std::ifstream in (filename.c_str(), std::ios_base::in);
    assert(in);

    for(size_t loc=0; loc < d_seqLen_p[chm]; loc++) {
      in >> letter;
      if(letter == '0') { //if unmappable
	d_mask_p[chm][loc] = false;
	incompCnt += (d_cntMappedTags_p[0][chm][loc] + d_cntMappedTags_p[1][chm][loc]);
      }
    }
  }
  if(incompCnt > 0) std::cout<<"Incompatible Counts between tag reads and mappability information = "<<incompCnt<<std::endl;
  return this;
}

SeqReadProto *Knk::NextPeak::readChipSeqMappedTags(
					     const char *inFileName_,
                                             const char *inFileName2_)
{
  std::ifstream in (inFileName_, std::ios_base::in);
  std::ifstream in2 (inFileName2_, std::ios_base::in);
  if(! in) Io::abortFormat("'%s' not found", inFileName_);
  if(inFileName2_ != 0)
    if(! in2) Io::abortFormat("'%s' not found", inFileName2_);

  return readChipSeqMappedTags(in, in2);
}

SeqReadProto * Knk::NextPeak::readChipSeqMappedTags(
					      std::ifstream &inFileStream_, // uniquely mapped reads
					      std::ifstream &inFileStream2_) // non-uniquely mapped reads (optional)
{
  assert(inFileStream_);
  clearChipSeqMappedTags();

  if(d_seqLen_p == 0) {
    d_seqLen_p = new long [d_seqNum];
    for(size_t chm = 0; chm < d_seqNum; chm++) {
      d_seqLen_p[chm] = getStrLen(d_strIndexFromChrom_p[chm]);
    }
  }

  d_cntMappedTags_p = new short **[2];
  for(size_t i = 0; i < 2; i++) {
    d_cntMappedTags_p[i] = new short *[d_seqNum];
    for(size_t chm = 0; chm < d_seqNum; chm++)
      d_cntMappedTags_p[i][chm] = new short [d_seqLen_p[chm]];
  }
  
  for(size_t i = 0; i < 2; i++)
    for(size_t chm = 0; chm < d_seqNum; chm++)
      for(size_t loc = 0; loc < d_seqLen_p[chm]; loc++)
        d_cntMappedTags_p[i][chm][loc] = 0;
  
  d_mask_p = new bool *[d_seqNum];
  for(size_t chm = 0; chm < d_seqNum; chm++)
    d_mask_p[chm] = new bool [d_seqLen_p[chm]];
  for(size_t chm = 0; chm < d_seqNum; chm++) 
    for(size_t loc = 0; loc < d_seqLen_p[chm]; loc++) 
      d_mask_p[chm][loc] = true;          //mappable


  char *read , strnd;
  size_t chromNum, loc;
  std::string line; //, read;                                                      
  char * lineStr;

  // input file format                      
  //  2 files could begenerated by Bowtie                
  //   file for uniquely mapped reads:                                   
  //     +       1  101670478   
  //   file for multiply mapped reads:           
  //     7  81056065 


  // for uniquely mapped reads

  while(inFileStream_ >> strnd >> chromNum >> loc) {
    if(strnd == '+') {
      d_cntMappedTags_p[0][chromNum-1][loc]++; //[loc-1]
    } else if(strnd == '-') {
      d_cntMappedTags_p[1][chromNum-1][loc]++; //[loc-1]
    }

  }

  // for multiply mapped reads  
  if(inFileStream2_ != 0) {
    while(inFileStream2_ >> chromNum >> loc)
      d_mask_p[chromNum-1][loc-1] = false;
  }

  return this;
}


bool Knk::NextPeak::useAnchoredDataEstBetaSigma(
				      double& beta_,    // initial value and return value
				      double& sigma_,   // initial value and return value
				      const char *inFileName_,
				      const double tol_) // tolerance level in Brent method
{
  long cntTotal = 0;
  if(inFileName_) {
    std::ifstream in (inFileName_, std::ios_base::in);
    if(! in) Njs::Io::abortFormat("'%s' not found", inFileName_);  

    d_cntAnchored.resize(2);
    d_cntAnchored[0].clear(); 
    d_cntAnchored[1].clear();
    long tmp1, tmp2, tmp3;
    while(in >> tmp1 >> tmp2 >> tmp3) {
      d_cntAnchored[0].push_back(tmp2);
      d_cntAnchored[1].push_back(tmp3);
      cntTotal += (tmp2+tmp3);
    }
  } else {
    for(size_t i = 0; i < d_cntAnchored[0].size(); i++)
      cntTotal += (d_cntAnchored[0][i] + d_cntAnchored[1][i]);
  }
  double halfCntTotal = (double)cntTotal * 0.5;
  long len = d_cntAnchored[0].size();
  long halflen = len * 0.5, ind;

  double sumf, negloglik, tolmu = 0.01, tolnu = 0.01, tolsigma =0.01, tolbeta =0.01;//, tolrho = 0.01;
  int cnt, error = 0;

  double normexp1[len], normexp2[len];
  double mu, muold, nu, nuold;
  double beta, betaold, sigma, sigmaold, rho;
  double ax_beta = beta_ * 0.1;
  double bx_beta = beta_;
  double cx_beta = beta_ * 10.0;
  double ax_sigma = sigma_* 0.1;
  double bx_sigma = sigma_;
  double cx_sigma = sigma_ * 10.0;

  double ax_mu = halflen * 0.9;
  double bx_mu = halflen;
  double cx_mu = halflen * 1.1;
  double ax_nu = cntTotal * 0.05;
  double bx_nu = cntTotal * 0.4;
  double cx_nu = cntTotal * 0.5;
  double lambda1[len], lambda2[len];
  mu = bx_mu;
  nu = bx_nu; 
  rho = (halfCntTotal - nu)/len;
  beta = bx_beta;
  sigma = bx_sigma;
  cnt = 0;    

  do {
    int iter;
    double a,b,d,etemp,fu,fv,fw,fx,p,q,r,tol1,tol2,u,v,w,x,xm;
    double e=0.0;

    muold = mu;
    nuold = nu;
    betaold = beta;
    sigmaold = sigma;

    Knk::Normmix::normexp(beta,sigma,len,normexp1,normexp2);

    // for mu
    a=(ax_mu < cx_mu ? ax_mu : cx_mu);
    b=(ax_mu > cx_mu ? ax_mu : cx_mu);
    x=w=v=bx_mu;

    sumf = 0.0;
    for(long j = 0; j < len; j++) {
      ind = j - (long)x + halflen;
      if( (ind >= 0) && (j - (long)x < halflen)) { 
	lambda1[j] = nu * normexp1[ind] +  rho;
	lambda2[j] = nu * normexp2[ind] +  rho;
	sumf += (normexp1[ind] + normexp2[ind]);
      }
      else {
	lambda1[j] = lambda2[j] = rho;
      }
    }
    negloglik = nu * sumf;
    for(size_t j = 0; j < len; j++) {
      negloglik -= (d_cntAnchored[0][j] * log(lambda1[j]) + d_cntAnchored[1][j] * log(lambda2[j]) );
    }

    fw=fv=fx= negloglik;      
    for (iter=1;iter<=ITMAX;iter++) {
      xm=0.5*(a+b);
      tol2=2.0*(tol1=tol_*fabs(x)+ZEPS);
      if (fabs(x-xm) <= tolmu){ 
	mu = x;
	break;
      }
      if (fabs(e) > tol1) {
	r=(x-w)*(fx-fv);
	q=(x-v)*(fx-fw);
	p=(x-v)*q-(x-w)*r;
	q=2.0*(q-r);
	if (q > 0.0) p = -p;
	q=fabs(q);
	etemp=e;
	e=d;
	if (fabs(p) >= fabs(0.5*q*etemp) || p <= q*(a-x) || p >= q*(b-x))
	  d=CGOLD*(e=(x >= xm ? a-x : b-x));
	else {
	  d=p/q;
	  u=x+d;
	  if (u-a < tol2 || b-u < tol2)
	    d=SIGN(tol1,xm-x);
	}
      } else {
	d=CGOLD*(e=(x >= xm ? a-x : b-x));
      }
      u=(fabs(d) >= tol1 ? x+d : x+SIGN(tol1,d));

      sumf = 0.0;
      for(long j = 0; j < len; j++) {
	ind = j - (long)u + halflen;
	if( (ind >= 0) && (j - (long)u < halflen)) {
	  lambda1[j] = nu * normexp1[ind] +  rho;
	  lambda2[j] = nu * normexp2[ind] +  rho;
	  sumf += (normexp1[ind] + normexp2[ind]);
	}
	else {
	  lambda1[j] = lambda2[j] = rho;
	}
      }
      negloglik = nu * sumf;
      for(size_t j = 0; j < len; j++) {
	negloglik -= (d_cntAnchored[0][j] * log(lambda1[j]) + d_cntAnchored[1][j] * log(lambda2[j]) );
      }
      
      fu= negloglik;                     
      if (fu <= fx) {
	if (u >= x) a=x; else b=x;
	SHFT(v,w,x,u)
	  SHFT(fv,fw,fx,fu)
	  } else {
	if (u < x) a=u; else b=u;
	if (fu <= fw || w == x) {
	  v=w;
	  w=u;
	  fv=fw;
	  fw=fu;
	} else if (fu <= fv || v == x || v == w) {
	  v=u;
	  fv=fu;
	}
      }
    }
    mu =x;
    
    // for nu
    e = 0.0;
    a=(ax_nu < cx_nu ? ax_nu : cx_nu);
    b=(ax_nu > cx_nu ? ax_nu : cx_nu);
    x=w=v=bx_nu;

    sumf = 0.0;  
    for(long j = 0; j < len; j++) {
      ind = j - (long)mu + halflen;
      if( (ind >= 0) && (j - (long)mu < halflen)) {
	lambda1[j] = x * normexp1[ind] +  rho;
	lambda2[j] = x * normexp2[ind] +  rho;
	sumf += (normexp1[ind] + normexp2[ind]);
      }
      else {
	lambda1[j] = lambda2[j] = rho;
      }
    }
    
    negloglik = x * sumf;
    for(size_t j = 0; j < len; j++) {
      negloglik -= (d_cntAnchored[0][j] * log(lambda1[j]) + d_cntAnchored[1][j] * log(lambda2[j]) );
    }
    
    fw=fv=fx= negloglik;  
    
    for (iter=1;iter<=ITMAX;iter++) {
      xm=0.5*(a+b);
      tol2=2.0*(tol1=tol_*fabs(x)+ZEPS);
      if (fabs(x-xm) <= tolnu){
	nu = x;
	break;
      }
      if (fabs(e) > tol1) {
	r=(x-w)*(fx-fv);
	q=(x-v)*(fx-fw);
	p=(x-v)*q-(x-w)*r;
	q=2.0*(q-r);
	if (q > 0.0) p = -p;
	q=fabs(q);
	etemp=e;
	e=d;
	if (fabs(p) >= fabs(0.5*q*etemp) || p <= q*(a-x) || p >= q*(b-x))
	  d=CGOLD*(e=(x >= xm ? a-x : b-x));
	else {
	  d=p/q;
	  u=x+d;
	  if (u-a < tol2 || b-u < tol2)
	    d=SIGN(tol1,xm-x);
	}
      } else {
	d=CGOLD*(e=(x >= xm ? a-x : b-x));
      }
      u=(fabs(d) >= tol1 ? x+d : x+SIGN(tol1,d));
      
      for(long j = 0; j < len; j++) {
	ind = j - (long)mu + halflen;
	if( (ind >= 0) && (j - (long)mu < halflen)) {
	  lambda1[j] = u * normexp1[ind] +  rho;
	  lambda2[j] = u * normexp2[ind] +  rho;
	}
	else {
	  lambda1[j] = lambda2[j] = rho;
	}
      }
      
      negloglik = u * sumf;
      for(size_t j = 0; j < len; j++) {
	negloglik -= (d_cntAnchored[0][j] * log(lambda1[j]) + d_cntAnchored[1][j] * log(lambda2[j]) );
      }
      
      fu= negloglik;
      
      if (fu <= fx) {
	if (u >= x) a=x; else b=x;
	SHFT(v,w,x,u)
	  SHFT(fv,fw,fx,fu)
	  } else {
	if (u < x) a=u; else b=u;
	if (fu <= fw || w == x) {
	  v=w;
	  w=u;
	  fv=fw;
	  fw=fu;
	} else if (fu <= fv || v == x || v == w) {
	  v=u;
	  fv=fu;
	}
      }
    }
    nu = x;
    rho = (halfCntTotal - nu)/len;
    if(rho < 1e-8) rho = 1e-8;

    //for beta
    a=(ax_beta < cx_beta ? ax_beta : cx_beta);
    b=(ax_beta > cx_beta ? ax_beta : cx_beta);
    x=w=v=bx_beta;

    Knk::Normmix::normexp(x,sigma,len,normexp1,normexp2);

    negloglik = 0.0;
    sumf = 0.0;
    for(long j = 0; j < len; j++) {
      ind = j - (long)mu + halflen;
      if( (ind >= 0) && (j - (long)mu < halflen)) {
	lambda1[j] = nu * normexp1[ind] +  rho;
	lambda2[j] = nu * normexp2[ind] +  rho;
	sumf += (normexp1[ind] + normexp2[ind]);
      }
      else {
	lambda1[j] = lambda2[j] = rho;
      }
    }

    negloglik += (nu * sumf + 2.0 * len * rho);
    for(size_t j = 0; j < len; j++) {
      negloglik -= (d_cntAnchored[0][j] * log(lambda1[j]) + d_cntAnchored[1][j] * log(lambda2[j]) );
    }
	
    fw=fv=fx= negloglik;

    for (iter=1;iter<=ITMAX;iter++) {
      xm=0.5*(a+b);
      tol2=2.0*(tol1=tol_*fabs(x)+ZEPS);
      if (fabs(x-xm) <= tolbeta){
	beta = x;
	break;
      }
      if (fabs(e) > tol1) {
	r=(x-w)*(fx-fv);
	q=(x-v)*(fx-fw);
	p=(x-v)*q-(x-w)*r;
	q=2.0*(q-r);
	if (q > 0.0) p = -p;
	q=fabs(q);
	etemp=e;
	e=d;
	if (fabs(p) >= fabs(0.5*q*etemp) || p <= q*(a-x) || p >= q*(b-x))
	  d=CGOLD*(e=(x >= xm ? a-x : b-x));
	else {
	  d=p/q;
	  u=x+d;
	  if (u-a < tol2 || b-u < tol2)
	    d=SIGN(tol1,xm-x);
	}
      } else {
	d=CGOLD*(e=(x >= xm ? a-x : b-x));
      }
      u=(fabs(d) >= tol1 ? x+d : x+SIGN(tol1,d));

      Knk::Normmix::normexp(u,sigma,len,normexp1,normexp2);
      
      negloglik = 0.0;
      sumf = 0.0;
      for(long j = 0; j < len; j++) {
	ind = j - (long)mu + halflen;
	if( (ind >= 0) && (j - (long)mu < halflen)) {
	  lambda1[j] = nu * normexp1[ind] +  rho;
	  lambda2[j] = nu * normexp2[ind] +  rho;
	  sumf += (normexp1[ind] + normexp2[ind]);
	}
	else {
	  lambda1[j] = lambda2[j] = rho;
	}
      }
      
      negloglik += (nu * sumf + 2.0 * len * rho);
      for(size_t j = 0; j < len; j++) {
	negloglik -= (d_cntAnchored[0][j] * log(lambda1[j]) + d_cntAnchored[1][j] * log(lambda2[j]) );
      }
      
      fu= negloglik;

      if (fu <= fx) {
	if (u >= x) a=x; else b=x;
	SHFT(v,w,x,u)
	  SHFT(fv,fw,fx,fu)
	  } else {
	if (u < x) a=u; else b=u;
	if (fu <= fw || w == x) {
	  v=w;
	  w=u;
	  fv=fw;
	  fw=fu;
	} else if (fu <= fv || v == x || v == w) {
	  v=u;
	  fv=fu;
	}
      }
    }
    beta = x;

    //for sigma
    e = 0.0;
    a=(ax_sigma < cx_sigma ? ax_sigma : cx_sigma);
    b=(ax_sigma > cx_sigma ? ax_sigma : cx_sigma);
    x=w=v=bx_sigma;
    Knk::Normmix::normexp(beta,x,len,normexp1,normexp2);
    negloglik = 0.0;
    sumf = 0.0;
    for(long j = 0; j < len; j++) {
      ind = j - (long)mu + halflen;
      if( (ind >= 0) && (j - (long)mu < halflen)) {
	lambda1[j] = nu * normexp1[ind] +  rho;
	lambda2[j] = nu * normexp2[ind] +  rho;
	sumf += (normexp1[ind] + normexp2[ind]);
      }
      else {
	lambda1[j] = lambda2[j] = rho;
      }
    }
    
    negloglik += (nu * sumf + 2.0 * len * rho);
    for(size_t j = 0; j < len; j++) {
      negloglik -= (d_cntAnchored[0][j] * log(lambda1[j]) + d_cntAnchored[1][j] * log(lambda2[j]) );
    }

    fw=fv=fx= negloglik;

    for (iter=1;iter<=ITMAX;iter++) {
      xm=0.5*(a+b);
      tol2=2.0*(tol1=tol_*fabs(x)+ZEPS);
      if (fabs(x-xm) <= tolsigma){
	sigma = x;
	break;
      }
      if (fabs(e) > tol1) {
	r=(x-w)*(fx-fv);
	q=(x-v)*(fx-fw);
	p=(x-v)*q-(x-w)*r;
	q=2.0*(q-r);
	if (q > 0.0) p = -p;
	q=fabs(q);
	etemp=e;
	e=d;
	if (fabs(p) >= fabs(0.5*q*etemp) || p <= q*(a-x) || p >= q*(b-x))
	  d=CGOLD*(e=(x >= xm ? a-x : b-x));
	else {
	  d=p/q;
	  u=x+d;
	  if (u-a < tol2 || b-u < tol2)
	    d=SIGN(tol1,xm-x);
	}
      } else {
	d=CGOLD*(e=(x >= xm ? a-x : b-x));
      }
      u=(fabs(d) >= tol1 ? x+d : x+SIGN(tol1,d));
      Knk::Normmix::normexp(beta,u,len,normexp1,normexp2);
     
      negloglik = 0.0;
      sumf = 0.0;
      for(long j = 0; j < len; j++) {
	ind = j - (long)mu + halflen;
	if( (ind >= 0) && (j - (long)mu < halflen)) {
	  lambda1[j] = nu * normexp1[ind] +  rho;
	  lambda2[j] = nu * normexp2[ind] +  rho;
	  sumf += (normexp1[ind] + normexp2[ind]);
	}
	else {
	  lambda1[j] = lambda2[j] = rho;
	}
      }
	
      negloglik += (nu * sumf + 2.0 * len * rho);
      for(size_t j = 0; j < len; j++) {
	negloglik -= (d_cntAnchored[0][j] * log(lambda1[j]) + d_cntAnchored[1][j] * log(lambda2[j]) );
      }
      
      fu= negloglik;

      if (fu <= fx) {
	if (u >= x) a=x; else b=x;
	SHFT(v,w,x,u)
	  SHFT(fv,fw,fx,fu)
	  } else {
	if (u < x) a=u; else b=u;
	if (fu <= fw || w == x) {
	  v=w;
	  w=u;
	  fv=fw;
	  fw=fu;
	} else if (fu <= fv || v == x || v == w) {
	  v=u;
	  fv=fu;
	}
      }
    }
    sigma = x;
  	
    // updating bounds
    cnt++;
    ax_mu = 0.5 * ax_mu + 0.5 * mu;
    bx_mu = mu;
    cx_mu = 0.5 * mu + 0.5 * cx_mu;
    bx_nu = nu;
    cx_nu = 0.1 * nu + 0.9 * cx_nu;
    ax_beta = 0.9 * ax_beta + 0.1 * beta;
    bx_beta = beta;
    cx_beta = 0.1 * beta + 0.9 * cx_beta;
    ax_sigma = 0.9 * ax_sigma + 0.1 * sigma;
    bx_sigma = sigma;
    cx_sigma = 0.1 * sigma + 0.9 * cx_sigma;

  } while((fabs(beta - betaold) > tolbeta || fabs(sigma - sigmaold) > tolsigma || fabs(mu - muold) > tolmu || fabs(nu - nuold) > tolnu) && cnt <=ITMAX);

  // print out
  std::cout<<"Parameter estimation using anchored data:";
  std::cout<<" beta="<<beta<<" sigma="<<sigma<<" mu="<<mu<<" nu="<<nu<<" rho="<<rho<<" Total count="<<cntTotal<<std::endl;

  // return values using arguments
  beta_ = beta;
  sigma_ = sigma;

  return true;

}

bool Knk::NextPeak::findReportPeaks(
				const double beta_,
				const double sigma_,
				const long densitylen_, // length of the density vector
				const long tagLen_,     // tag length
				const long motifLen_,   // motif length
				const long distMotSite_,// minimum distance to motif site to be counted
				const bool find2ndPeaks_,
				const bool useBIC_, 
				const int binNum_, // bin number per strand for goodness of fit test
				const double tol_)  // tolerance level in Brent method
{
  int halflenf = densitylen_*0.5, ind, mappableLen, leftCnt;
  double mu, muold, nu, nuold, rho, sumf,sumf_1,sumf_2, negloglik, negloglikold, tolmu =0.01, tolnu =0.01, negloglik0,rho0,TS_nu,pval;
  int cnt, error = 0;

  double normexp1[densitylen_], normexp2[densitylen_], dmu_normexp1[densitylen_], dmu_normexp2[densitylen_];

  Knk::Normmix::normexp(beta_,sigma_,densitylen_,normexp1,normexp2,dmu_normexp1,dmu_normexp2);

  // Goodness of Fit precalculation
  double confLevel = 0.95;
  double alpha = 1.0 - confLevel;
  int lowerLeft, upperLeft, lowerRight, upperRight; // relative to the center, e.g, -90, 20, -20, 90
  double tmp1 = 0, tmp2 = 0;
  bool lowerL=true, upperL=true, lowerR=true, upperR=true;
  for(size_t i = 0; i < densitylen_; i++) {
    tmp1 += normexp1[i]; tmp2 += normexp2[i];
    if(lowerL && (tmp1 > alpha)) {
      lowerLeft = i - halflenf;
      lowerL = false;
    }
    if(upperL && (tmp1 > 1 - alpha)) {
      upperLeft = i - halflenf;
      upperL = false;
    }
    if(lowerR && (tmp2 > alpha)) {
      lowerRight = i - halflenf;
      lowerR = false;
    }
    if(upperR && (tmp2 > 1 - alpha)) {
      upperRight = i  - halflenf;
      upperR = false;
    }
  }
  
  int intMu, df;
  bool binMappable;
  double Y, Yhat, TS_GoF, pvalTS_GoF, lambda1sum, lambda2sum, cumsum, halfCnt;

  size_t chr, regionLen, maxRegionLen = 0, m;

  // initializing vectors
  const size_t regionNum = d_regionChrom.size();

  for(size_t i =0; i < regionNum; i++)
    if(d_regionRightEnds[i] - d_regionLeftEnds[i] > maxRegionLen)
      maxRegionLen = d_regionRightEnds[i] - d_regionLeftEnds[i];

  double lambda1[maxRegionLen], lambda2[maxRegionLen],lambda1_1[maxRegionLen], lambda2_1[maxRegionLen],lambda1_2[maxRegionLen], lambda2_2[maxRegionLen], EZ1_1[maxRegionLen], EZ2_1[maxRegionLen], EZ1_2[maxRegionLen], EZ2_2[maxRegionLen],EZ1_0[maxRegionLen], EZ2_0[maxRegionLen];

    double laminv1, laminv2, dmu_dlam1, dmu_dlam2, dmu2_d2lam1, dmu2_d2lam2, y_lamsq1, y_lamsq2 ;
    double Ednusq_d2loglik = 0, Edmusq_d2loglik = 0, Ednumu_d2loglik = 0; 
    double nuvar, muvar, Det;

  std::vector<int> v_regionIndex, v_mappableLen, v_leftCnt ,v_df;
  std::vector<double> v_mu,v_muStdErr,v_2nu,v_2nuStdErr,v_rho,v_rhoRatio,v_pval,v_dev,v_pvalTS_GoF;
  double mu1, muold1, mu2, muold2, nu1, nuold1, nu2, nuold2, sumEZ_1, sumEZ_2, rhoMulti; 

  // Loop for each region
  for(size_t i = 0; i < regionNum; i++) {
    chr = d_regionChrom[i];
    regionLen = d_regionRightEnds[i] - d_regionLeftEnds[i];

    mappableLen = leftCnt = 0;

    for(size_t j = d_regionLeftEnds[i]; j < d_regionRightEnds[i]; j++) {
      if(d_mask_p[chr][j]) {
	leftCnt += d_cntMappedTags_p[0][chr][j];
	mappableLen ++;
      }
    }
    halfCnt = (double)d_regionCnt[i] * 0.5;

    rho0 =  halfCnt / mappableLen; 
    double ax_mu = 0.0;
    double bx_mu = regionLen * 0.5;
    double cx_mu = regionLen - 1.0;
    double ax_nu = 0.0;
    double bx_nu = d_regionCnt[i]*0.4;
    double cx_nu = d_regionCnt[i];
    mu = muold = bx_mu;
    nu = nuold = bx_nu;
    rho = (halfCnt - nu)/mappableLen;
    cnt = 0;

    do {
      int iter;
      double a,b,d,etemp,fu,fv,fw,fx,p,q,r,tol1,tol2,u,v,w,x,xm;
      double e=0.0;
      muold = mu;
      nuold = nu;

      // for mu
      a=(ax_mu < cx_mu ? ax_mu : cx_mu);
      b=(ax_mu > cx_mu ? ax_mu : cx_mu);
      x=w=v=bx_mu;

      sumf = 0.0;
      for(long j = 0; j < regionLen; j++) {
        if(d_mask_p[chr][d_regionLeftEnds[i]+j]) {
          ind = j - (long)x + halflenf;
          if( (ind >= 0) && (j - (long)x < halflenf)) {  //change of location of equality
            lambda1[j] = nu * normexp1[ind] +  rho;
            lambda2[j] = nu * normexp2[ind] +  rho;
            sumf += (normexp1[ind] + normexp2[ind]);
          }
          else {
            lambda1[j] = lambda2[j] = rho;
          }
        }
      }


      negloglik = nu * sumf;
      for(size_t m = d_regionLeftEnds[i]; m < d_regionRightEnds[i]; m++) {
        if(d_mask_p[chr][m]) negloglik -= (d_cntMappedTags_p[0][chr][m] * log(lambda1[m-d_regionLeftEnds[i]]) + d_cntMappedTags_p[1][chr][m] * log(lambda2[m-d_regionLeftEnds[i]]) );
      }

      fw=fv=fx= negloglik;
      for (iter=1;iter<=ITMAX;iter++) {
        xm=0.5*(a+b);
        tol2=2.0*(tol1=tol_*fabs(x)+ZEPS);
        if (fabs(x-xm) <= tolmu){
          mu = x;
          break;
        }
        if (fabs(e) > tol1) {
          r=(x-w)*(fx-fv);
          q=(x-v)*(fx-fw);
          p=(x-v)*q-(x-w)*r;
          q=2.0*(q-r);
          if (q > 0.0) p = -p;
          q=fabs(q);
          etemp=e;
          e=d;
          if (fabs(p) >= fabs(0.5*q*etemp) || p <= q*(a-x) || p >= q*(b-x))
            d=CGOLD*(e=(x >= xm ? a-x : b-x));
          else {
            d=p/q;
            u=x+d;
            if (u-a < tol2 || b-u < tol2)
              d=SIGN(tol1,xm-x);
          }
        } else {
          d=CGOLD*(e=(x >= xm ? a-x : b-x));
        }
        u=(fabs(d) >= tol1 ? x+d : x+SIGN(tol1,d));

        sumf = 0.0;
        for(long j = 0; j < regionLen; j++) {
          if(d_mask_p[chr][d_regionLeftEnds[i]+j]) {
            ind = j - (long)u + halflenf;
            if( (ind >= 0) && (j - (long)u < halflenf)) {
              lambda1[j] = nu * normexp1[ind] +  rho;
              lambda2[j] = nu * normexp2[ind] +  rho;
	      sumf += (normexp1[ind] + normexp2[ind]);
	    }
	      else {
		lambda1[j] = lambda2[j] = rho;
	      }
	    }
	  }


	  negloglik = nu * sumf;
	  for(size_t m = d_regionLeftEnds[i]; m < d_regionRightEnds[i]; m++) {
	    if(d_mask_p[chr][m]) negloglik -= (d_cntMappedTags_p[0][chr][m] * log(lambda1[m-d_regionLeftEnds[i]]) + d_cntMappedTags_p[1][chr][m] * log(lambda2[m-d_regionLeftEnds[i]]) );
	  }

	  fu= negloglik;
	  if (fu <= fx) {
	    if (u >= x) a=x; else b=x;
	    SHFT(v,w,x,u)
	      SHFT(fv,fw,fx,fu)
	      } else {
	    if (u < x) a=u; else b=u;
	    if (fu <= fw || w == x) {
	      v=w;
	      w=u;
	      fv=fw;
	      fw=fu;
	    } else if (fu <= fv || v == x || v == w) {
	      v=u;
	      fv=fu;
	    }
	  }
	}
	mu =x;

	// for nu
	e=0.0;
	a=(ax_nu < cx_nu ? ax_nu : cx_nu);
	b=(ax_nu > cx_nu ? ax_nu : cx_nu);
	x=w=v=bx_nu;

	for(long j = 0; j < regionLen; j++) {
	  if(d_mask_p[chr][d_regionLeftEnds[i]+j]) {
	    ind = j - (long)mu + halflenf;
	    if( (ind >= 0) && (j - (long)mu < halflenf)) {
	      lambda1[j] = x * normexp1[ind] +  rho;
	      lambda2[j] = x * normexp2[ind] +  rho;
	    }
	    else {
	      lambda1[j] = lambda2[j] = rho;
	    }
	  }
	}

	negloglik = x * sumf;
	for(size_t m = d_regionLeftEnds[i]; m < d_regionRightEnds[i]; m++) {
	  if(d_mask_p[chr][m]) negloglik -= (d_cntMappedTags_p[0][chr][m] * log(lambda1[m-d_regionLeftEnds[i]]) + d_cntMappedTags_p[1][chr][m] * log(lambda2[m-d_regionLeftEnds[i]]) );
	}

	fw=fv=fx= negloglik;

	for (iter=1;iter<=ITMAX;iter++) {
	  xm=0.5*(a+b);
	  tol2=2.0*(tol1=tol_*fabs(x)+ZEPS);
	  if (fabs(x-xm) <= tolnu){ 
	    nu = x;
	    break;
	  }
	  if (fabs(e) > tol1) {
	    r=(x-w)*(fx-fv);
	    q=(x-v)*(fx-fw);
	    p=(x-v)*q-(x-w)*r;
	    q=2.0*(q-r);
	    if (q > 0.0) p = -p;
	    q=fabs(q);
	    etemp=e;
	    e=d;
	    if (fabs(p) >= fabs(0.5*q*etemp) || p <= q*(a-x) || p >= q*(b-x))
	      d=CGOLD*(e=(x >= xm ? a-x : b-x));
	    else {
	      d=p/q;
	      u=x+d;
	      if (u-a < tol2 || b-u < tol2)
		d=SIGN(tol1,xm-x);
	    }
	  } else {
	    d=CGOLD*(e=(x >= xm ? a-x : b-x));
	  }
	  u=(fabs(d) >= tol1 ? x+d : x+SIGN(tol1,d));

	  for(long j = 0; j < regionLen; j++) {
	    if(d_mask_p[chr][d_regionLeftEnds[i]+j]) {
	      ind = j - (long)mu + halflenf;
	      if( (ind >= 0) && (j - (long)mu < halflenf)) {
		lambda1[j] = u * normexp1[ind] +  rho;
		lambda2[j] = u * normexp2[ind] +  rho;
	      }
	      else {
		lambda1[j] = lambda2[j] = rho;
	      }
	    }
	  }

	  negloglik = u * sumf;
	  for(size_t m = d_regionLeftEnds[i]; m < d_regionRightEnds[i]; m++) {
	    if(d_mask_p[chr][m]) negloglik -= (d_cntMappedTags_p[0][chr][m] * log(lambda1[m-d_regionLeftEnds[i]]) + d_cntMappedTags_p[1][chr][m] * log(lambda2[m-d_regionLeftEnds[i]]) );
	  }


	  fu= negloglik;

	  if (fu <= fx) {
	    if (u >= x) a=x; else b=x;
	    SHFT(v,w,x,u)
	      SHFT(fv,fw,fx,fu)
	      } else {
	    if (u < x) a=u; else b=u;
	    if (fu <= fw || w == x) {
	      v=w;
	      w=u;
	      fv=fw;
	      fw=fu;
	    } else if (fu <= fv || v == x || v == w) {
	      v=u;
	      fv=fu;
	    }
	  }
	}
	nu = x;
	rho = (halfCnt - nu*sumf*0.5)/mappableLen;
	if(rho < 1e-8) rho = 1e-8;

	// updating bounds
	cnt++;
	ax_mu = 0.5 * ax_mu + 0.5 * mu;
	bx_mu = mu;
	cx_mu = 0.5 * mu + 0.5 * cx_mu;
	bx_nu = nu;
	if(nu < 100)  cx_nu = 0.33 * nu + 0.67 * cx_nu;
	else cx_nu = (nu * 3 < cx_nu ? nu *3: 0.33 * nu + 0.67 * cx_nu);

    } while((fabs(mu - muold) > tolmu || fabs(nu - nuold) > tolnu ) && cnt <=ITMAX);

    // finding variance estimate based on Fisher information


    for(long j = 0; j < regionLen; j++) {
      ind = j - (long)mu + halflenf;
      if(d_mask_p[chr][d_regionLeftEnds[i]+j] && (ind >= 0) && (j - (long)mu < halflenf)) {
        lambda1[j] = nu * normexp1[ind] +  rho;
        lambda2[j] = nu * normexp2[ind] +  rho;
	laminv1 = 1.0 / lambda1[j];
        laminv2 = 1.0 / lambda2[j];
        dmu_dlam1 = nu * dmu_normexp1[ind];
        dmu_dlam2 = nu * dmu_normexp2[ind];
	Ednusq_d2loglik += (-normexp1[ind]*normexp1[ind] * laminv1 - normexp2[ind]*normexp2[ind] * laminv2 );
	Ednumu_d2loglik += (- normexp1[ind] *dmu_dlam1 * laminv1 - normexp2[ind] *dmu_dlam2 * laminv2 );
	Edmusq_d2loglik += (- dmu_dlam1 *dmu_dlam1 * laminv1 - dmu_dlam2 *dmu_dlam2 * laminv2 );
      }
    }

    Det = Edmusq_d2loglik * Ednusq_d2loglik - Ednumu_d2loglik * Ednumu_d2loglik;
    Det = 1.0 / Det;
    nuvar = -Edmusq_d2loglik * Det; 
    muvar = -Ednusq_d2loglik * Det; 

    // finding loglik of the null model
    negloglik0 = -(d_regionCnt[i] * (log(rho0) -1.0));
    TS_nu = 2.0 * (negloglik0 - negloglik);
    error = 0;
    pval = Njs::ChiSquare::rightTail(1.0, TS_nu, &error);

    // Goodness of Fit for each region
    intMu = (int)(mu + 0.5);
    TS_GoF = 0.0;
    df = 0;
    lambda1sum = lambda2sum = 0.0;
    for(int k = std::max(intMu + lowerLeft,0); k < intMu + upperLeft && k <regionLen; k ++) {
      if(d_mask_p[chr][d_regionLeftEnds[i]+k])
	lambda1sum += lambda1[k];
    }
    for(int k = std::max(intMu + lowerRight,0); k < intMu + upperRight && k <regionLen; k ++) {
      if(d_mask_p[chr][d_regionLeftEnds[i]+k])
	lambda2sum += lambda2[k];
    }
    cumsum = 0.0;
    for(int k = std::max(intMu + lowerLeft, 0), cnt =0; k < intMu + upperLeft && k <regionLen && cnt < binNum_; cnt++) {
      Y = Yhat = 0.0;
      binMappable = false;
      do {
	if(d_mask_p[chr][d_regionLeftEnds[i]+k]) {
	  Y += d_cntMappedTags_p[0][chr][d_regionLeftEnds[i]+k];
	  Yhat += lambda1[k];
	  cumsum += lambda1[k];
	  binMappable = true;
	}
	k++;
      } while(cumsum <(cnt+1.0)*lambda1sum/(double)binNum_ && k <regionLen);

      if(binMappable) {
	df++;
	if(Y > 0.5) TS_GoF += (Y * log(Y/Yhat) - Y + Yhat);
	else        TS_GoF += Yhat;
      }
    }     
    cumsum = 0.0;
    for(int k = std::max(intMu + lowerRight, 0), cnt=0; k < intMu + upperRight && k <regionLen && cnt < binNum_; cnt++) {
      Y = Yhat = 0.0;
      binMappable = false;
      do {
	if(d_mask_p[chr][d_regionLeftEnds[i]+k]) {
	  Y += d_cntMappedTags_p[1][chr][d_regionLeftEnds[i]+k];
	  Yhat += lambda2[k];
	  cumsum += lambda2[k];
	  binMappable = true;
	}
	k++;
      } while(cumsum <(cnt+1.0)*lambda2sum/(double)binNum_ && k <regionLen);
      
      if(binMappable) {
	df++;
	if(Y > 0.5) TS_GoF += (Y * log(Y/Yhat) - Y + Yhat);
	else        TS_GoF += Yhat;
      }
    }     

    TS_GoF = TS_GoF * 2.0;
    if(df > 2)
      df -= 2; // df = n - k where k is the number of parameters in the reduced model (mu, nu)
    else {
      df = 1;
      TS_GoF = 0/0;
    }
    error = 0;
    pvalTS_GoF = Njs::ChiSquare::rightTail(df, TS_GoF, &error);
    
  
    // saving output for a regular region
    v_regionIndex.push_back(i);
    v_mappableLen.push_back(mappableLen);
    v_mu.push_back(mu);
    v_muStdErr.push_back(sqrt(muvar));
    v_leftCnt.push_back(leftCnt);
    v_2nu.push_back(2.0*nu);
    v_2nuStdErr.push_back(2.0*sqrt(nuvar));
    v_rho.push_back(rho);
    v_rhoRatio.push_back(1-rho/rho0);
    v_pval.push_back(pval);
    v_dev.push_back(TS_GoF);
    v_df.push_back(df);
    v_pvalTS_GoF.push_back(pvalTS_GoF);

  }


  // finding cutoff values for region length and p-value  
  const long subregionNum = v_regionIndex.size();
  const int pvalExpLow=2, pvalExpHigh=16, pvalExpInc=2, regLenLow = 100, regLenHigh = 600, regLenInc = 100, grpSize = 50, grpNum =200; 
  int grpFreq[grpNum], tmp, arg_n, arg_m;
  double grpWeight[grpNum], tmpmax, base10 = 10.0;
  bool isMotifNear[subregionNum], regionSelect[subregionNum], restart=true;
  int pvalNum = (pvalExpHigh - pvalExpLow)/pvalExpInc + 1;
  int regLenNum = (regLenHigh - regLenLow)/regLenInc + 1;
  double AUC[pvalNum][regLenNum];
  size_t prevDist, curDist, curMotifInd = 0, prevMotifChm, rank[subregionNum];
  const size_t motifLocSize = d_motifLoc.size();
  size_t k = 0;

  if(motifLocSize > 0) {
  for(size_t i=0; i < subregionNum; i++) 
    d_motifLoc[i] -= ((tagLen_ - motifLen_)* 0.5);

  for(size_t m=0; m < pvalNum; m++)
    for(size_t n=0; n < regLenNum; n++)
      AUC[m][n] = 0.0;
  for(size_t i=0; i < subregionNum; i++) isMotifNear[i] = false;

  // assigning values to isMotifNear vector
  for(size_t i=0; i < subregionNum ; i++) {
    while (curMotifInd < motifLocSize) {
      curDist = abs(d_regionLeftEnds[v_regionIndex[i]] + (long)(v_mu[i] + 0.5) - d_motifLoc[curMotifInd]);
      if(restart) {
	prevMotifChm = d_regionChrom[v_regionIndex[i]];
	prevDist = curDist;
	restart = false;
	if(d_regionChrom[v_regionIndex[i]] < d_motifChm[curMotifInd]) {
	  while(d_regionChrom[v_regionIndex[i]] < d_motifChm[curMotifInd]) i++;
	  i--;
	  break;
	}
	if(d_regionChrom[v_regionIndex[i]] > d_motifChm[curMotifInd])
	  while(d_regionChrom[v_regionIndex[i]] > d_motifChm[curMotifInd] && curMotifInd < motifLocSize-1) curMotifInd++;
	else curMotifInd++;
	continue;
      }
      if(prevMotifChm < d_motifChm[curMotifInd]) {
	while(d_regionChrom[v_regionIndex[i]] < d_motifChm[curMotifInd]){ 
	  if(abs(d_regionLeftEnds[v_regionIndex[i]] + (long)(v_mu[i] + 0.5) - d_motifLoc[curMotifInd-1]) <= distMotSite_){ 
	      isMotifNear[i] = true;
	  }
	  i++;
	}
	i--;	
	restart = true;
	break;
      } else if(d_regionChrom[v_regionIndex[i]] > d_motifChm[curMotifInd]) {
	restart = true;
	continue;
      } else if(prevDist <= curDist) { // on the same chromosome
	if(prevDist <= distMotSite_) {
	  if(!isMotifNear[i-1] || d_regionLeftEnds[v_regionIndex[i]] != d_regionLeftEnds[v_regionIndex[i-1]])
	    isMotifNear[i] = true;
	}	  
	restart = true;
	curMotifInd--;
	break;
      } else {
	prevDist = curDist;
	curMotifInd++;
      }
    }
   }
   k=0;
    for(size_t i=0; i < subregionNum; i++)
      if(isMotifNear[i]) {
	 k++; 
      }

  //finding AUC values
  for(size_t g=0; g < grpNum; g++) grpWeight[g] = 0.0;
  for(size_t g=0; g < grpNum; g++)
    for(long h=0; h <= g; h++)
      grpWeight[h] += (1/((double)grpSize*(g+1)));

  for(size_t n=0; n <regLenNum; n++) {
    for(size_t i=0; i < subregionNum; i++) {
      if(d_regionRightEnds[v_regionIndex[i]]- d_regionLeftEnds[v_regionIndex[i]] >= regLenLow + n*regLenInc || v_pvalTS_GoF[i] >= 1e-6) regionSelect[i] = true;
      else regionSelect[i] = false;
    }
    for(size_t i=0; i < subregionNum; i++)
      if(regionSelect[i]) {
	rank[i]=0;  //rank starts from 0
      }
    
    for(size_t i=0; i < subregionNum; i++) {
      if(regionSelect[i]) {	
	k = 0;
	for(size_t j=0; j < subregionNum; j++) 
	  if(regionSelect[j] && v_2nu[i] < v_2nu[j]) k++;
	rank[i] = k;
      }
    }
    for(size_t g=0; g < grpNum; g++) grpFreq[g] = 0;
    for(size_t i=0; i < subregionNum; i++)
      if(regionSelect[i] && isMotifNear[i] && rank[i] < grpNum*grpSize) 
	(grpFreq[rank[i]/grpSize])++;
    for(size_t g=0; g < grpNum; g++) {
      AUC[2][n] += (grpFreq[g] * grpWeight[g]);  // fixing p-value cutoff = 1e-6;
    }
  }
  tmpmax = 0.0;
  for(size_t n=0; n < regLenNum; n++) 
    if(tmpmax < AUC[2][n]) {
      tmpmax = AUC[2][n];
      arg_n = n;
    }

  for(size_t m=0; m < pvalNum; m++) {
    if(AUC[m][arg_n] == 0.0) {
    for(size_t i=0; i < subregionNum; i++) {
      if(d_regionRightEnds[v_regionIndex[i]]- d_regionLeftEnds[v_regionIndex[i]] >= regLenLow + arg_n*regLenInc  || v_pvalTS_GoF[i] >= pow(base10,(int)(-(pvalExpLow + m*pvalExpInc)))) regionSelect[i]=true;
      else regionSelect[i] = false;
    }      
    for(size_t i=0; i < subregionNum; i++)
      if(regionSelect[i]) {
	rank[i]=0;
      }
    for(size_t i=0; i < subregionNum; i++) {
      if(regionSelect[i]) {	
	k = 0;
	for(size_t j=0; j < subregionNum; j++) 
	  if(regionSelect[j] && v_2nu[i] < v_2nu[j]) k++;
	rank[i] = k;
      }
    }    
    for(size_t g=0; g < grpNum; g++) grpFreq[g] = 0;
    for(size_t i=0; i < subregionNum; i++)
      if(regionSelect[i] && isMotifNear[i] && rank[i] < grpNum*grpSize) 
	(grpFreq[rank[i]/grpSize])++;
    
    for(size_t g=0; g < grpNum; g++) 
      AUC[m][arg_n] += (grpFreq[g] * grpWeight[g]);  // fixing region length cutoff    

    }
  }

  tmpmax = 0.0;
  for(size_t m=0; m < pvalNum; m++)  
    if(tmpmax < AUC[m][arg_n]) {
      tmpmax = AUC[m][arg_n];
      arg_m = m;
    }

  do {
    for(size_t n=max(0,arg_n -1); n <=min(arg_n+1,regLenNum-1); n++) {
      for(size_t m=max(0,arg_m -1); m <= min(arg_m +1,pvalNum-1); m++) {
	if(AUC[m][n] == 0.0) {
	  for(size_t i=0; i < subregionNum; i++) {
	    if(d_regionRightEnds[v_regionIndex[i]]- d_regionLeftEnds[v_regionIndex[i]] >= regLenLow + n*regLenInc  || v_pvalTS_GoF[i] >= pow(base10,(int)(-(pvalExpLow + m*pvalExpInc)))) regionSelect[i]=true;
	    else regionSelect[i] = false;
	  }      
	  for(size_t i=0; i < subregionNum; i++)
	    if(regionSelect[i]) {
	      rank[i]=0;
	    }
	  for(size_t i=0; i < subregionNum; i++) {
	    if(regionSelect[i]) {	
	      k = 0;
	      for(size_t j=0; j < subregionNum; j++) 
		if(regionSelect[j] && v_2nu[i] < v_2nu[j]) k++;
	      rank[i] = k;
	    }
	  }    
	  for(size_t g=0; g < grpNum; g++) grpFreq[g] = 0;
	  for(size_t i=0; i < subregionNum; i++)
	    if(regionSelect[i] && isMotifNear[i] && rank[i] < grpNum*grpSize) 
	      (grpFreq[rank[i]/grpSize])++;
	  
	  for(size_t g=0; g < grpNum; g++) 
	    AUC[m][n] += (grpFreq[g] * grpWeight[g]);
	  
	}
      } 
    }

    tmpmax = 0.0;
    for(size_t n=0; n < regLenNum; n++) 
      for(size_t m=0; m < pvalNum; m++)  
	if(tmpmax < AUC[m][n]) {
	  tmpmax = AUC[m][n]; arg_m = m; arg_n = n;
	}

  } while(AUC[max(0,arg_m -1)][max(0,arg_n -1)]==0.0 || AUC[min(arg_m +1,pvalNum-1)][min(arg_n+1,regLenNum-1)]==0.0 || AUC[max(0,arg_m -1)][min(arg_n+1,regLenNum-1)]==0.0 || AUC[min(arg_m +1,pvalNum-1)][max(0,arg_n -1)]==0.0 );

  std::cout<<"cutoff for spurious region: (region length = "<<regLenLow + arg_n*regLenInc<<", p-value for GOF = 1e-"<<pvalExpLow + arg_m*pvalExpInc<<") AUC = "<<tmpmax/grpNum<<" Distance to motif cutoff="<<distMotSite_<<std::endl;

  } // end of if(motifLocSize > 0) 

  // print out
  for(size_t i=0; i < subregionNum; i++) 
    std::cout<<d_regionChrom[v_regionIndex[i]]+1<<" "<<d_regionLeftEnds[v_regionIndex[i]]+1<<" "<<d_regionRightEnds[v_regionIndex[i]]<<" "<<d_regionRightEnds[v_regionIndex[i]]-d_regionLeftEnds[v_regionIndex[i]]<<" "<<v_mappableLen[i]<<" "<<(double)v_mappableLen[i]/(d_regionRightEnds[v_regionIndex[i]]-d_regionLeftEnds[v_regionIndex[i]])<<" "<<v_mu[i]<<" "<<v_muStdErr[i]<<" "<<v_leftCnt[i]<<" "<<(double)v_leftCnt[i]/d_regionCnt[v_regionIndex[i]]<<" "<<d_regionCnt[v_regionIndex[i]]<<" "<<v_2nu[i]<<" "<<v_2nuStdErr[i]<<" "<<v_rho[i]<<" "<<v_2nuStdErr[i]*0.5/v_mappableLen[i]<<" "<<v_rhoRatio[i]<<" "<<v_pval[i]<<" "<<v_dev[i]<<" "<<v_df[i]<<" "<<v_pvalTS_GoF[i]<<std::endl;


  return true;
}



#undef ITMAX
#undef CGOLD
#undef ZEPS
#undef SHFT
#undef SIGN
