// knk_nextpeak.h

#ifndef INCLUDED_KNK_NEXTPEAK
#define INCLUDED_KNK_NEXTPEAK

#include <njs_seqread.h>
#include <njs_io.h>

using namespace Njs;

namespace Knk {

  class NextPeak : public SeqRead {

  public:
    inline NextPeak ()
      :d_seqNum(0), d_seqLen_p(0), d_chromNum_p(0), d_cntMappedTags_p(0), d_strIndexFromChrom_p(0), d_mask_p(0) {}

    virtual ~NextPeak() {free();}

    virtual SeqReadProto *setChromNum(); //defline of the input fasta file has to be single or two digit number following immediately after '>',e.g.,">23", ">5"
    // also works for defline for NBCI 36.1 of form ">gi|224589823|ref|NC_000024.9| Homo sapiens ... " and UCSC hg18 of form ">chr1" or ">chrY" 
    virtual SeqReadProto *setStrIndexFromChrom();
    virtual SeqReadProto *readSeqLenFile(const char* inFileName_);
    virtual SeqReadProto *clearChipSeqMappedTags ();
    virtual SeqReadProto *free ();
    
    virtual SeqReadProto *readChipSeqMappedTags(std::ifstream &inFileStream_, 
						 std::ifstream &inFileStream2_);
    virtual SeqReadProto *readChipSeqMappedTags(const char *inFileName_, //uniquely mapped reads 
						 const char *inFileName2_ = 0); // non-uniquely mapped reads (optional)

    virtual SeqReadProto *readMappability(const char *inDirName_); // directory name should end with '/', e.g., /home/mappability/
    virtual SeqReadProto *readMappability(std::string inDirName_); // file names are expected to be chr(numbers).map, e.g., chr1.map,... ,chr24.map
    virtual inline long getSeqNum() const {return d_seqNum;}
    virtual inline long getSeqLen(size_t i_) const {return d_seqLen_p[i_];}
    virtual inline long getChromNum(size_t i_) const {return d_chromNum_p[i_];}
    virtual inline long getStrIndexFromChrom(size_t i_) const {return d_strIndexFromChrom_p[i_];}

    virtual inline long getCntMappedTags(
       const size_t strand_,    // forward (0), backward(1)
       const size_t chromNum_,  // starts from 0
       const size_t loc_) const // starts from 0
    {return d_cntMappedTags_p[strand_][chromNum_][loc_];}

    virtual inline bool getMask(  //getting mappabiling info: mappable(1), unmappable(0)
       const size_t chromNum_,  // starts from 0
       const size_t loc_) const // starts from 0
    {return d_mask_p[chromNum_][loc_];}

    virtual inline long getRegionChrom(size_t i_) const {return d_regionChrom[i_];} // i_th region
    virtual inline long getRegionLeftEnds(size_t i_) const {return d_regionLeftEnds[i_];}
    virtual inline long getRegionRightEnds(size_t i_) const {return d_regionRightEnds[i_];}
    virtual inline long getRegionCnt(size_t i_) const {return d_regionCnt[i_];} 
    virtual inline long getRegionSize() const {return d_regionChrom.size();}

    virtual inline long getCntAnchored(
       const size_t strand_,    // forward (0), backward(1)
       const size_t loc_) const // starts from 0
    {return d_cntAnchored[strand_][loc_];}

    virtual inline long getMotifChm(size_t i_) const {return d_motifChm[i_];}
    virtual inline long getMotifLoc(size_t i_) const {return d_motifLoc[i_];}

    virtual inline long getWinSize() const {return d_winSize;}
    virtual inline long getMinReadsPerRegion() const {return d_minReadsPerRegion;}
    virtual inline long getReadLen() const {return d_readLen;}

    virtual SeqReadProto *setWinSizeMinTagNum(
       long winSize_, 
       long minReadsPerRegion_);
    virtual SeqReadProto *writeRegions(const char *outfile_);
    virtual SeqReadProto *writeRegions(std::ofstream &outStream_);
    virtual SeqReadProto *readSimRegions(const char *inFileName_);
    void useHg18SeqLen();
    void findRegions();
    bool reportRegions();
    void readMotifLocReportAnchoredLoc(
				       const char *inFileName_, 
				       const long width_, 
				       const long shift_,
				       const char *outFileName_=NULL);
    bool useAnchoredDataEstBetaSigma(
				      double& beta_,    // initial vaulue and return value
				      double& sigma_,   // initial vaulue and return value
				      const char *inFileName_ = NULL, // may be given from an external file				      
				      const double tol_ = 0.001); // tolerance level in Brent method

    bool findReportPeaks(
			   const double beta_,
			   const double sigma_,
			   const long densitylen_, // length of the density vector 
			   const long tagLen_,     // tag length
			   const long motifLen_,   // motif length
			   const long distMotSite_,// minimum distance to motif site to be counted
			   const bool find2ndPeaks_=false, 
			   const bool useBIC_=true,
			   const int binNum_ = 7,  // bin number per strand for goodness of fit test
			   const double tol_  = 0.001);  // tolerance level in Brent method
  private:

    long d_seqNum;
    long * d_seqLen_p;
    long * d_chromNum_p;
    long * d_strIndexFromChrom_p;
    short *** d_cntMappedTags_p; //unsigned char
    bool **d_mask_p;
    std::map <std::string, int> d_tooManyMatchRead;

    std::vector <long> d_regionChrom;
    std::vector <long> d_regionLeftEnds;
    std::vector <long> d_regionRightEnds;
    std::vector <long> d_regionCnt;

    std::vector <std::vector<long> > d_cntAnchored;
    std::vector <long> d_motifChm, d_motifLoc;

    long d_winSize;
    long d_minReadsPerRegion;
    long d_readLen;

    long d_curCnt;
    long d_curChrom;
    long d_curLeft;
    long d_curRight;

    long cnt(
       const long chrom_,
       const long left_);
    long cnt(
       const long chrom_,
       const long left_,
       const long right_);
    long cntNext();
    long cntPrev();
    long cntIncreasingWin();
  };
}

#endif //! INCLUDED_KNK_NEXTPEAK
