// knk_normmix.cpp

#include <knk_normmix.h>
#include <njs_normal.h>

using namespace Knk;

double Normmix::cdfN(double zscore_)
{
  int error = 0;
  return (1.0 - Njs::Normal::rightTail(zscore_, &error));
}

int Normmix::normexp(
	    double beta_,
	    double sigma_,
	    size_t len_,
	    double * densityLeft_,
	    double * densityRight_,
	    double * derivativeLeft_,
	    double * derivativeRight_)

{
  double halflen = (double)(len_) * 0.5;
  double beta_inv = 1.0 / beta_, sigma_inv = 1.0 / sigma_;
  double surv[len_];
  int error = 0;

  for(long i=0; i < len_; i++) {
    surv[i] = Njs::Normal::rightTail((i - halflen +0.5 + sigma_*sigma_*beta_inv)*sigma_inv, &error);
    densityLeft_[i] = surv[i]  * exp((i - halflen +0.5  + 0.5 * sigma_*sigma_*beta_inv)*beta_inv) * beta_inv;
  }
  densityRight_[0] = Njs::Normal::rightTail((len_ - halflen +0.5 + sigma_*sigma_*beta_inv)*sigma_inv, &error) * exp((len_ - halflen +0.5  + 0.5 * sigma_*sigma_*beta_inv)*beta_inv) * beta_inv;
  for(long i=1; i < len_; i++) {
    densityRight_[i] = densityLeft_[len_-i];
  }

  if(derivativeLeft_ != 0 && derivativeRight_!= 0) {
    const double sqrt2PI = sqrt(2.0 * 3.1415927);
    for(long i=0; i < len_; i++) {
      derivativeLeft_[i] = (exp(-0.5*pow((i - halflen +0.5 +sigma_*sigma_*beta_inv)*sigma_inv, 2)) /  (surv[i] * sqrt2PI * sigma_) - beta_inv ) * densityLeft_[i];
    }
    derivativeRight_[0] = -(exp(-0.5*pow((len_ - halflen +0.5 +sigma_*sigma_*beta_inv)*sigma_inv, 2)) /  ( Njs::Normal::rightTail((len_ - halflen +0.5 + sigma_*sigma_*beta_inv)*sigma_inv, &error)* sqrt2PI * sigma_) - beta_inv ) * densityRight_[0];
    for(long i=1; i < len_; i++) {
      derivativeRight_[i] = - derivativeLeft_[len_-i];
    }
  }
  return 1;
}

int Normmix::normexp(
	    double beta_,
	    double sigma_,
	    size_t len_,
	    int direction_, //1 for the support of Expnential being on the left, 2 on the right
	    double * values_)
{
  double halflen = (double)(len_)*0.5;
  double beta_inv = 1.0 / beta_, sigma_inv = 1.0 / sigma_;
  if (direction_ == 1) {
    for(long i=0; i < len_; i++) {
      values_[i] = (1.0 - cdfN((i - halflen +0.5 + sigma_*sigma_*beta_inv)*sigma_inv) ) * exp((i - halflen +0.5  + 0.5 * sigma_*sigma_*beta_inv)*beta_inv) * beta_inv;
    }
  }      
  else if(direction_ == 2) {
    for(long i=0; i < len_; i++) {
      values_[i] = cdfN((i - halflen +0.5  - sigma_*sigma_*beta_inv)*sigma_inv) * exp( -(i - halflen +0.5  - 0.5 * sigma_*sigma_*beta_inv)*beta_inv) * beta_inv;
    }
  }
  
  return 1;
}

