// njs_combinatoric.h

#ifndef INCLUDED_NJS_COMBINATORIC
#define INCLUDED_NJS_COMBINATORIC

#ifndef INCLUDED_NJS_SYSTEM
#include <njs_system.h>
#endif

#ifndef INCLUDED_NJS_IO
#include <njs_io.h>
#endif

/*****************************************************************************
 
************************** COMBINATORIAL ROUTINES ****************************
  
*****************************************************************************/

namespace Njs {

   namespace Combinatoric {

      template <class Integer>
      Integer factorial (                // (Integer) factorial
      Integer n)
      {
	      if (n < 0) {
      Njs::Io::abort ("Combinatoric::factorial <Integer> : negative argument");
	      }
	      Integer iFactorial = 1;
	      for (Integer i = 2; i <= n; i++) {
	         iFactorial *= i;
	      }
      return iFactorial;
      }

      template <class Integer>
      Integer binomial (                  // (Integer) binomial coefficient 
      Integer n,
      Integer r)
      {
	      if (r < 0 || r > n || n < 0)
      return 0;
         r = n - r < r ? n - r : r;
	      Integer ibico = 1;
	      for (Integer i = 0; i < r; i++) {
		      ibico = ibico * (n - i) / (i + 1);
	      }
	   return ibico;
      }

      template <class Real, class Integer>
      Integer realBinomial (               // (Real, Integer) binomial coefficient 
      Real x,
      Integer r)
      {
	      if (r < 0)
      return 0.0;
	      Real xBinomial = 1.0;
	      for (Integer i = 1; i <= r; x -= 1.0, i++) {
	         xBinomial *= x / i;
	      }
      return xBinomial;
      }

      template <class Integer>
      Integer multinomial (              // (Integer) multinomial coefficient
      Integer n, 
      Integer *r) 
      {
	      Integer ibico = 1;
	      Integer rsum = r[0];
	      for (Integer i = 1; i < n; i++) {
		      rsum += r[i];
		      ibico *= binomial <Integer> (rsum, r[i]);
	      }
	   return ibico;
      }

   }

}

#endif //! INCLUDED_NJS_COMBINATORIC

