// njs_io.h

#ifndef INCLUDED_NJS_IO
#define INCLUDED_NJS_IO

#ifndef INCLUDED_NJS_SYSTEM
#include <njs_system.h>
#endif

namespace Njs {

   namespace Io {

      int appendAndCloseFile ( // opens, appends, and closes file
      const char *nameOut, // output file : status : "a"
      const char *format, // format string
      ...);

      int writeAndCloseFile ( // opens, overwrites, and closes file
      const char *nameOut, // output file : status : "a"
      const char *format, // format string
      ...);

      inline int eraseFile ( // opens, erases, and closes file
      const char *nameOut) // output file : status : "a"
      {return writeAndCloseFile (nameOut, "");};

      void abortFormat ( // aborts with a formatted message
      const char *format, // format string
      ...);

      void abort ( // aborts with a string
      const char *message); // error message

      char *changePostfix ( // changes the postfix of a string
      char *out, // output name
      const char *dat, // input name
      const char *outPostFix = 0, // output post-fix (0 = "out")
      const char *datPostFix_ = 0, // input post-fix (0 = "dat")
      char separator_ = '\0'); // separator ('\0' = '.')
      // assumes sufficient storage in out

      int outStream ( // appends file
      std::ostream &out_, // output stream
      const char *format, // format string
      ...);

      int outStream ( // appends file
      std::ostream *out_, // output stream pointer
      const char *format, // format string
      ...);

      template <class T>
      void outVector (
      std::ostream &out_, // output stream
      const char *fmtName, // format for vector name
      const char *name, // vector name
      const char *fmt, // format for each vector element
      const T *vec, // vector
      const long m, const long n) // m <= j < n                       
      {
         outStream (out_, fmtName, name);
         for (long j = m; j < n; j++) {
            outStream (out_, fmt, (T) vec [j]);
         }
      }

      template <class T>
      void outMatrix (
      std::ostream &out_, // output stream
      const char *fmtName, // format for vector name
      const char *name, // vector name
      const char *fmt, // format for each vector element
      const T *const *mat, // matrix
      const long m1, const long n1, // m1 <= j1 < n1 
      const long m2, const long n2, // m2 <= j2 < n2  
      const bool transpose = false)
      {
         outStream (out_, fmtName, name);
         if (transpose) {
            for (long j2 = m2; j2 < n2; j2++) {
               for (long j1 = m1; j1 < n1; j1++) {
                  outStream (out_, fmt, (T) mat [j1][j2]);
               }
               if (j2 < n2) outStream (out_, "\n");
            }
         } else {
            for (long j1 = m1; j1 < n1; j1++) {
               for (long j2 = m2; j2 < n2; j2++) {
                  outStream (out_, fmt, (T) mat [j1][j2]);
               }
               if (j1 < n1 - 1) outStream (out_, "\n");
            }
         }
      }

      template <class T>
      void outTriMatrix (
      std::ostream &out_, // output stream
      const char *fmtName, // format for vector name
      const char *name, // vector name
      const char *fmt, // format for each vector element
      const T *const *const *tri,// trimatrix
      const long m1, const long n1, // m1 <= j1 < n1 
      const long m2, const long n2, // m2 <= j2 < n2 
      const long m3, const long n3) // m3 <= j3 < n3
      {
         outStream (out_, fmtName, name);
         for (long j1 = m1; j1 < n1; j1++) {
            for (long j2 = m2; j2 < n2; j2++) {
               for (long j3 = m3; j3 < n3; j3++) {
                  outStream (out_, fmt, (T) tri [j1][j2][j3]);
               }
               if (j1 < n1 - 1 || j2 < n2 - 1) outStream (out_, "\n");
            }
            if (j1 < n1 - 1) outStream (out_, "\n");
         }
      }

 // String-based routines

      bool readLineToT ( // assumes buf can handle one line of input
      std::istream &in, // input filestream
      size_t dimBuf, // #(buf)
      char *buf, // string before '!' from next line
      const char *t = "!"); // termination characters (not white space)

      void stringToDouble ( // reads single double values from a string
      char *str, // input : string
      double *x); // output : double value 

      void stringToLong ( // reads single real values from a string
      char *str, // input : string
      long *x); // output : long value 

   };

}

#endif //! INCLUDED_NJS_IO