// njs_memutil.h

#ifndef INCLUDED_NJS_MEMUTIL
#define INCLUDED_NJS_MEMUTIL

#ifndef INCLUDED_NJS_SYSTEM
#include <njs_system.h>
#endif

#ifndef INCLUDED_NJS_IO
#include <njs_io.h>
#endif

namespace Njs {

   namespace MemUtil {

      inline void *memNew (size_t size)
      { 
      return (void *) (size == 0 ? 0 : new char [size]);
      }

      inline void *memMore (void *p, size_t size)
      {
      return (void *) (p == 0 ? new char [size] : realloc (p, size));
      }

      inline void memCpy (void *to, const void *from, size_t size)
      { 
	      if (size != 0) memcpy (to, from, size);
      }

      inline void *memFree (void *p) 
      {
	      delete p;
      return (void *) 0;
      }

      inline void *memSet (void *p, int c, size_t size)
      {
      return memset (p, c, size);
      }

      inline void *memZero (void *p, size_t size)
      {
      return memset (p, '\0', size);
      }

      template <class T>
      T *create (const T &t_, bool isCopy_ = false) // creates if the class has a default and copy constructor
      {return isCopy_ ? new T (t_) : new T;}

      template <class T>
      T *newVector (long iMin, long iMax) // vector [iMin...iMax]
      {
         if (iMax < iMin) {
      Njs::Io::abortFormat ("iMax = %ld < %ld = iMin\n", (long) iMax, (long) iMin);
         }
      return (T *) ((new char [sizeof (T) * (size_t) (iMax - iMin + 1)]) - sizeof (T) * iMin);
      }

      template <class T>
      inline void deleteVector (T *&vector_, long iMin, long iMax)
      {
         delete [] ((char *) vector_ + sizeof (T) * iMin);
         vector_ = 0;
      }

      template <class T>
      T **newMatrix (long iMin, long iMax, long jMin, long jMax) // matrix [iMin...iMax][jMin...jMax]
      {
         if (iMax < iMin) {
      Njs::Io::abortFormat ("iMax = %ld < %ld = iMin\n", (long) iMax, (long) iMin);
         } else if (jMax < jMin) {
      Njs::Io::abortFormat ("jMax = %ld < %ld = jMin\n", (long) jMax, (long) jMin);
         }
         T **matrix = (new T * [(size_t) (iMax - iMin + 1)]) - iMin;
         for (long index = iMin; index <= iMax; index++) {
            matrix [index] = (T *) ((new char [sizeof (T) * (size_t) (jMax - jMin + 1)]) - sizeof (T) * jMin);
         }
      return matrix;
      }

      template <class T>
      void deleteMatrix (T **&matrix, long iMin, long iMax, long jMin, long jMax) 
      // Assumes matrix was created with the call 
      //         matrix = newMatrix <T> (iMin, iMax, jMin, jMax);
      {
         for (long index = iMin; index <= iMax; index++) {
            delete [] ((char *) matrix [index] + sizeof (T) * jMin);
         }
         delete [] (matrix + iMin);
         matrix = 0;
      }

   }

}

#endif //! INCLUDED_NJS_MEMUTIL
