// njs_normal.cpp

#include <njs_normal.h>

#undef NJS_DEBUG
#include <njs_debug.h>

#include <njs_float.h>
#include <njs_function.h>
#include <njs_io.h>
#include <njs_rootfinder.h>
#include <njs_uniform.h>
#include <njs_variate.h>

using namespace Njs;

double Normal::standardVariate () // returns RV normal (0, 1)
{
   static bool logGot = false;
	static double gotVariate;
	double factor, v1, v2, rsquare;
	if (logGot) {
		logGot = false;
return gotVariate;
	} else {
		logGot = true;
		do {
         v1 = Uniform::variate (-1.0, 1.0);
         v2 = Uniform::variate (-1.0, 1.0);
			rsquare = v1 * v1 + v2 * v2;
		} while (rsquare > 1.0); 
		factor = sqrt (-2.0 * log (rsquare) / rsquare);
		gotVariate = factor * v1;
return factor * v2;
	}
}

static double l_normalRightTail (         // Normal right tail 
double x,
int *error,
double rtol)
{
   if (*error != 0) {
return 0.0;
   }
   if (x == 0.0) {
return 0.5;
   } else if (x < 0.0) {
return 0.5 * (1.0 + Function::erf (-x / Njs::Float::ROOT_2, error, rtol));
   } else {
return 0.5 * Function::erfc (x / Njs::Float::ROOT_2, error, rtol);
   }
}

double Normal::rightTail (
double x,                           // argument
int *error,                        // error = 0 : error-free return
double rtol,                        // tolerance 
double mu,                          // mean
double sigma)                       // standard deviation
{
return sigma <= 0.0 ? Variate::BAD_PROBABILITY : l_normalRightTail ((x - mu) / sigma, error, rtol);
}

double Normal::density (           // Normal density 
double x)
{
return exp (- 0.5 * x * x) / Float::ROOT_2_PI;
}

static int *s_error_NormalRightTail = 0;
static double s_rtol_NormalRightTail = 0.0;

static double l_normalRightTail_s (double x)  // Normal right tail 
{
   int error = 0;
   double y = l_normalRightTail (x, &error, s_rtol_NormalRightTail);
   if (*s_error_NormalRightTail == 0) {
      *s_error_NormalRightTail = error;
   }
return y;
}

double Normal::sigLevel (              // Gaussian significance level 
double p,
int *error,
double tol)
{
   const double EPS = 1.0e-06;
   if (*error != 0) {
return 0.0;
   }
   s_error_NormalRightTail = error;
   s_rtol_NormalRightTail = tol;
	if (p <= 0.0) { 
Njs::Io::abort ("Function::Normal::sigLevel : p <= 0.0");
	} else if (1.0 <= p) {
Njs::Io::abort ("Function::Normal::sigLevel : 1.0 <= p");
	}
   double sign = p < 0.5 ? 1.0 : -1.0;
	if (p == 0.5) {
return 0.0;
	} else if (p > 0.5) {
		p = 1.0 - p;
	}
	double x = sqrt (-2.0 * log (2.0 * p));
return sign * 
   RootFinder::newtonRaphson (p, l_normalRightTail_s, Normal::density, 0.0, 0.5 * x, x, tol);
}

