// njs_seqread.cpp

#include <njs_seqread.h>

#include <njs_memutil.h>

#undef NJS_DEBUG
#include <njs_debug.h>

using namespace Njs;

SeqReadProto *SeqRead::free ()
{
   if (CharPtrPtrVec::size () != 0) {
      for (CharPtrPtrVec::const_iterator pos = CharPtrPtrVec::end () - 1; pos >= CharPtrPtrVec::begin (); pos--) {
         delete [] (*pos) [0]; (*pos) [0] = 0;
         delete [] (*pos) [1]; (*pos) [1] = 0;
         delete [] (*pos);
      }
      CharPtrPtrVec::clear ();
   }
   d_strlen.clear ();
return this;
}

void SeqRead::push_back (char *seq_) // pushes back a copy of a new sequence with blank defline
{
   if (seq_ == 0 || *seq_ == '\0') return;
   char **seq = new char * [2];
   seq [0] = new char [1];
   seq [0][0] = '\0'; // def-line
   seq [1] = seq_;
   push_back (seq);
   d_strlen.push_back (strlen (seq [1]));
   delete seq [0]; seq [0] = 0;
   delete seq; seq = 0;
}
      
void SeqRead::push_back (char **seq_) // pushes back a copy of a defline and sequence
{
   if (seq_ == 0 || *seq_ [1] == '\0') return;
   char **seq = new char * [2];
   seq [0] = new char [strlen (seq_ [0]) + 1]; // def-line
   strcpy (seq [0], seq_ [0]);
   seq [1] = new char [strlen (seq_ [1]) + 1];
   strcpy (seq [1], seq_ [1]);
// add to vector 
   CharPtrPtrVec::push_back (seq);
   d_strlen.push_back (strlen (seq [1]));
}


SeqReadProto *SeqRead::addFastaBuffer (
const char *buffer_, // buffer of characters
bool isDefLine_) // ? Is the buffer a defline ? 
{ // adds a new buffer to the sequences
   if (buffer_ == 0 || *buffer_ == '\0') {
return this;
   }
   if (isDefLine_) {
      char **seq = new char * [2];
	  // def-line
      if (d_storeDefLine) {
         seq [0] = new char [strlen (buffer_) + 2];
         strcpy (seq [0], ">"); // def-line start
         strcat (seq [0], buffer_); // def-line
      } else {
         seq [0] = new char [1];
         seq [0][0] = '\0'; // def-line
      }
	  // sequence line
      seq [1] = new char [1];
      seq [1][0] = '\0'; // sequence "start"
	  // add to vector 
      CharPtrPtrVec::push_back (seq);
      d_strlen.push_back (0);
   } else {
      CharPtrPtrVec::back () [1] = (char *) Njs::MemUtil::memMore (CharPtrPtrVec::back () [1], 
      sizeof (char) * (strlen (CharPtrPtrVec::back () [1]) + strlen (buffer_) + 1));
      strcat (CharPtrPtrVec::back () [1], buffer_);
      d_strlen.pop_back ();
      d_strlen.push_back (strlen (CharPtrPtrVec::back () [1]));
   }
return this;
}

SeqReadProto *SeqRead::addFastaFile (
std::ifstream &inFileStream_, //--input filestream : input in fasta format
const char *buffer_, //--sequence buffer
size_t dimBuffer_) //--dimension of the sequence buffer (including trailing '\0')
{
   assert (inFileStream_);
   enum STATUS {START, DEF_LINE, SEQ_LINE};
   STATUS status;
   char c;
   std::string sequence;
   std::string defline;
   //--fasta sequence file reader (NOTE: The method has a problem with input files that contain broken def-lines)
   while (inFileStream_ >> c) {
	   if (isalpha(c)) {
		   sequence += c;
	   } else if (c == '>') {
		   buffer_ = sequence.c_str();
		   status = SEQ_LINE;
		   addFastaBuffer (buffer_, status == DEF_LINE);
		   status = DEF_LINE;
		   std::getline(inFileStream_, defline);
		   buffer_ = defline.c_str();
		   //--erase the first carriage return character
		   //--should be taken care of when transferring files between DOS and UNIX.
		   char *carriage_return = strchr (buffer_, '\r');
		   if (carriage_return) *carriage_return = '\0';
		   addFastaBuffer (buffer_, status == DEF_LINE);
		   sequence = "";
	   } else {
		   //---do nothing
	   }
   }
   buffer_ = sequence.c_str();
   status = SEQ_LINE;
   addFastaBuffer (buffer_, status == DEF_LINE);

return this;
}


SeqReadProto *SeqRead::addFastaFile (
const char *inFileName_, //--filename : input in fasta format
const char *buffer_, //--sequence buffer
size_t dimBuffer_) //--dimension of the sequence buffer (including trailing '\0')
{
   std::ifstream in (inFileName_, std::ios_base::in); // input stream in fasta format
   if (! in) Io::abortFormat ("'%s' not found", inFileName_);
   addFastaFile (in, buffer_, dimBuffer_);
return this;
}

namespace {
   
   SeqReadOut n_out;

}

SeqReadOut *SeqReadOut::out = &n_out;

void SeqReadOut::clear () {out = &n_out;} // resets out state to default

std::ostream &SeqRead::out (std::ostream &ostr_)
const {
// begin output
   const size_t FIELD_WIDTH = 80;
   char *buffer = new char [FIELD_WIDTH + 1];
   size_t i = 0;
   for (CharPtrPtrVec::const_iterator pos = begin (); pos < end (); pos++, i++) {
      if (pos != begin ()) {
         ostr_ << "\n";
      }
      ostr_.width (0);
      ostr_ << (*pos) [0];
      if (SeqReadOut::out->outLength) {
         if ((*pos) [0][0] != '\0') ostr_ << " | ";
         ostr_ << "length = " << d_strlen [i];
      }
      ostr_.width (FIELD_WIDTH);
      for (char *buf = (*pos) [1]; buf < (*pos) [1] + strlen ((*pos) [1]); buf += FIELD_WIDTH) {
         ostr_ << "\n";
         strncpy (buffer, buf, FIELD_WIDTH);
         ostr_ << buffer;
      }
   }
   delete [] buffer; buffer = 0 ;
return ostr_;
}





