// njs_seqreadproto.h

#ifndef INCLUDED_NJS_SEQREADPROTO
#define INCLUDED_NJS_SEQREADPROTO

#include <njs_system.h>

/*****************************************************************************
 
************************* SEQUENCE READING ROUTINES **************************
  
*****************************************************************************/

namespace Njs {

   class HashProto;

   class SeqReadProto {

      public:

      virtual ~SeqReadProto ();

      virtual operator bool () const = 0; // ? does the SeqRead contain something?

      virtual SeqReadProto *addFastaFile (
      std::ifstream &inFileStream_, // input filestream : input in fasta format
      const char *buffer_, // sequence buffer
      size_t dimBuffer_) = 0;// dimension of the sequence buffer (including trailing '\0')
      // buffer_ MUST be able to hold the longest line in the input

      virtual SeqReadProto *addFastaFile (
      const char *inFileName_, // filename : input in fasta format
      const char *buffer_, // sequence buffer
      size_t dimBuffer_) = 0; // dimension of the sequence buffer (including trailing '\0')
      // buffer_ MUST be able to hold the longest line in the input

      virtual SeqReadProto *free () = 0; // clears the sequences and frees memory.

      virtual std::ostream &out (std::ostream &ostr_) const = 0; // output.

      virtual void push_back (char *seq_) = 0; // pushes back a copy of a new sequence with blank defline
      virtual void push_back (char **seq_) = 0; // pushes back a copy of a defline and sequence

      virtual SeqReadProto *initialize (HashProto *hash_) = 0; // performs initialization of the object after sequences are loaded
      virtual SeqReadProto *setStoreDefLine (bool storeDefLine_) = 0; // clears the sequences and frees memory.
      virtual char *getSeq (size_t i) const = 0; // pointer to the 0-th char in seq i
      virtual char *getSeqs (size_t i) const = 0; // pointer to the 0-th char in seq i

      virtual bool getStoreDefLine () const = 0; // ? store the def-line ?
      virtual const char *getDefLine (size_t i) const = 0; // pointer to the 0-th char in def-line i
	   virtual const size_t getCountSeqs () const = 0; // pointer to the number of sequences in the file.
      virtual std::vector <char **>::size_type size () const = 0; // extension of the std::vector <char **> function
      virtual std::vector <char **>::const_iterator begin () const = 0; // extension of the std::vector <char **> function
      virtual std::vector <char **>::const_iterator end () const = 0; // extension of the std::vector <char **> function

   };

}

inline std::ostream &operator<< (std::ostream &ostr_, const Njs::SeqReadProto *seqRead_)
{return seqRead_->out (ostr_);}

#endif //! INCLUDED_NJS_SEQREADPROTO











