// knk_chipseq.t.cpp

#undef NJS_DEBUG

#include <knk_args.h>
#include <knk_nextpeak.h>

#include <knk_normmix.h>

using namespace Njs;
using namespace Knk;


int main (int argc, char *argv[])
{
  args::parse(argc, argv); 
   NextPeak seq;
   seq.free ()->setStoreDefLine (true);
   
   if(args::genomicfile) {
     const size_t DIM_BUFFER = 1024 * 1024;
     char *buffer = new char [DIM_BUFFER];
     seq.addFastaFile(args::genomicfile, buffer, DIM_BUFFER); 
     delete [] buffer; buffer = 0;
     seq.setChromNum(); 
     seq.setStrIndexFromChrom();
   }

   if(args::seqlenfile)
     seq.readSeqLenFile(args::seqlenfile); 
   else 
     seq.useHg18SeqLen(); // default genome: hg18 (NCBI Build 36.1)
      
   seq.setWinSizeMinTagNum(args::winSize, args::minTagNum);

   seq.readChipSeqMappedTags(args::chipseqfile);
   std::cout<<"ChIP-seq file: "<<args::chipseqfile<<std::endl;

   if(args::mappabilitydir) {
     std::cout<<"Mappability information is used"<<std::endl;
     seq.readMappability(args::mappabilitydir);
   }

   seq.findRegions(); 

   if(args::regionOutFile) 
     seq.writeRegions(args::regionOutFile);
  
   if(args::motifLocFile) {
     std::cout<<"Motif location file: "<<args::motifLocFile<<std::endl;
     seq.readMotifLocReportAnchoredLoc(args::motifLocFile, args::tabVecLen, (args::tagLen - args::motifLen)/2, args::anchoredCntFile);
     seq.useAnchoredDataEstBetaSigma(args::beta, args::sigma);
   }

   seq.findReportPeaks(args::beta, args::sigma, args::tabVecLen,args::tagLen, args::motifLen, args::distMotSite); 
   
   //seq.free();

return 1;
}
